#ifndef __XRDOUCN2NLOADER_HH__
#define __XRDOUCN2NLOADER_HH__
/******************************************************************************/
/*                                                                            */
/*                    X r d O u c N 2 N L o a d e r . h h                     */
/*                                                                            */
/* (c) 2012 by the Board of Trustees of the Leland Stanford, Jr., University  */
/*                            All Rights Reserved                             */
/*   Produced by Andrew Hanushevsky for Stanford University under contract    */
/*              DE-AC02-76-SFO0515 with the Department of Energy              */
/*                                                                            */
/* This file is part of the XRootD software suite.                            */
/*                                                                            */
/* XRootD is free software: you can redistribute it and/or modify it under    */
/* the terms of the GNU Lesser General Public License as published by the     */
/* Free Software Foundation, either version 3 of the License, or (at your     */
/* option) any later version.                                                 */
/*                                                                            */
/* XRootD is distributed in the hope that it will be useful, but WITHOUT      */
/* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or      */
/* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public       */
/* License for more details.                                                  */
/*                                                                            */
/* You should have received a copy of the GNU Lesser General Public License   */
/* along with XRootD in a file called COPYING.LESSER (LGPL license) and file  */
/* COPYING (GPL license).  If not, see <http://www.gnu.org/licenses/>.        */
/*                                                                            */
/* The copyright holder's institutional names and contributor's names may not */
/* be used to endorse or promote products derived from this software without  */
/* specific prior written permission of the institution or contributor.       */
/******************************************************************************/
  
#include "XrdOuc/XrdOucName2Name.hh"

class  XrdOucEnv;
struct XrdVersionInfo;

class XrdOucN2NLoader
{
public:

XrdOucName2Name *Load(const char *libName, XrdVersionInfo &urVer,
                      XrdOucEnv *envP=0);

                 XrdOucN2NLoader(XrdOucgetName2NameArgs)
                                : eRoute(eDest), cFN(confg),
                                  libParms((parms ? parms : "")),
                                  lclRoot(lroot), rmtRoot(rroot) {}
                ~XrdOucN2NLoader() {}

private:

XrdSysError *eRoute;
const char  *cFN;
const char  *libParms;
const char  *lclRoot;
const char  *rmtRoot;
};
#endif
